{
  hotlabel.pas
    v1.0 of 24/10/1999
      A label component descended from TLabel that stores a URL and starts the
      the default browser to display the web page or the email client to send
      an email to the URL.
}

{
  History
  - v1.0 of 24/10/1999
      Original version.
}

{
  Dependencies
      NONE
}

{
  Legal notice
    This component is copyright  P.D.Johnson, 1999.
    The source code and help files can be freely distributed on a not-for-profit
    basis providing that:
      + the source code is not altered.
      + this readme file is distributed with it unchanged
    By not-for-profit I mean that you may recover out of pocket expenses
    incurred in distributing the code, but should not make a profit from this.
    If you discover any bugs in this implementation, or if you have any update
    suggestions, please contact me on peter.johnson@openlink.org.
    Please do modify the code for you own use. I'd like to see any changes you
    make - I could incorporate them into future versions. Please notify me of
    changes on at the above e-mail address.
    This software is provided as is - no warranty is given as to its suitability
    for any purposes to which you may wish to put it.
}


unit HotLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const
  {Hand cursor used as default cursor}
  crHand = 10;

type
  {Exception triggered by TPJHotLabel when URL problems encountered}
  EPJURLError = class(Exception);

type
  {Label component that can use its caption text as a hotlink to default web
  browser or e-mail program}
  TPJHotLabel = class(TLabel)
  private {new properties}
    FValidateURL : Boolean;
    procedure SetValidateURL(const AFlag : Boolean);
  protected
    procedure SetDefaultURL; virtual;
      {Sets the caption text to a default URL. Override this method in
      descendant classes to change this value}
    procedure Click; override;
      {Overrides inherited Click method to start default browser or e-mail
      client if Enabled property is true}
    procedure CheckURL(const URL : string); virtual;
      {Checks the given URL for validity (i.e. has file:, http://, ftp:// or
      mailto: protocols) and raises exception if URL is not for one of these
      protocols. Over-ride this method to add additional protocols in a
      descendant component}
  protected {inherited properties}
    procedure SetCaption(const ACaption : string); virtual;
    function GetCaption : string; virtual;
  public
    constructor Create(AOwner : TComponent); override;
      {Override of inherited constructor to set appropriate defaults}
  published
    {New properties}
    property ValidateURL : Boolean read FValidateURL write SetValidateURL
      default True;
      {Causes Caption property to be checked for valid URLs when true, and
      does not check when false. Setting this property true when there is an
      invalid URL in the Caption throws an exception and restores the default
      URL to the Caption. Setting this property false means that the component
      acts as if it were a TLabel except for default font colour and style}
    {"Overridden" inherited properties}
    property Caption : string read GetCaption write SetCaption;
      {Caption property contains the URL. This is validated if ValidateURL
      property is true - raises exception if user sets Caption to an invalid
      URL}
    property Cursor default crHand;
      {Change default cursor of inherited Cursor property from crDefault to
      crHand}
    property ParentFont default False;
  end;

procedure Register;

implementation

uses
  ShellAPI;

{$R HOTLABEL.RES}

const
  CCursorName = 'HOTLABEL_HANDCUR';
    {Name of hand cursor resource}
  CDefaultURL = 'http://homepages.tesco.net/~p.d.johnson/';
    {URL used as default caption property by SetDefaultURL method}

constructor TPJHotLabel.Create(AOwner : TComponent);
  {Override of inherited constructor to set appropriate defaults}
begin
  inherited Create(AOwner);
  {Load hand cursor from resources and use it by default}
  Screen.Cursors[crHand] := LoadCursor(HInstance, CCursorName);
  Cursor := crHand;
  {Set default font colour and style - we have ParentFont false because of this}
  Font.Color := clNavy;
  Font.Style := [fsUnderline];
  {Ensure we're validating URLs by default}
  SetValidateURL(True);
  {Set default caption value to default URL}
  SetDefaultURL;
end;

procedure TPJHotLabel.SetDefaultURL;
  {Sets the caption text to a default URL. Override this method in descendant
  classes to change this value}
begin
  try
    {Try to set caption to default URL}
    SetCaption(CDefaultURL)
  except on EPJURLError do
    {Catch exception thrown if default caption isn't valid and set caption to ''
    instead - aren't we being cutious!}
    SetCaption('');
  end;
end;

procedure TPJHotLabel.Click;
  {Overrides inherited Click method to start default browser or e-mail client if
  Enabled property is true}
var
  URL : string;   {the URL we're going to access}
begin
  {Get URL from caption property}
  URL := GetCaption;
  {Only try to access URL if it's not empty string and if component is enabled}
  if Enabled and (URL <> '')then
  begin
   {Get windows to decode URL and execute default browser or e-mail client}
    if ShellExecute(ValidParentForm(Self).Handle, nil, PChar(URL), nil,
      nil, SW_SHOW) < 32 then
      {... and raise excpetion if there was a problem}
      raise EPJURLError.CreateFmt(
        'Can''t activate browser or e-mail program for URL "%s"', [URL]);
  end;
  {Now, if we got thru all that successfully, call inherited click stuff}
  inherited Click;
end;

procedure TPJHotLabel.CheckURL(const URL : string);
  {Checks the given URL for validity (i.e. has file:, http://, ftp:// or mailto:
  protocols) and raises exception if URL is not for one of these protocols.
  Over-ride this method to add additional protocols in a descendant component}
const
  {These are the URLs we recognise}
  CValidURLS : array[1..4] of string[12]
    = ('http://', 'mailto', 'file:', 'ftp://');
var
  I : Integer;  {loops thru all recognised URLs}
begin
  {Allow ''  we don't ever try to juimp to this so it causes no problems}
  if URL = '' then Exit;
  {Check caption against array of valid protocols and exit if we recognise one}
  for I := 1 to 4 do
    if CompareText(CValidURLS[I],Copy(URL, 1, Length(CValidURLs[I]))) = 0 then
      Exit;
  {If we get here, we haven't recognised the protocol so raise exception}
  raise EPJURLError.CreateFmt('Protocol not recognised for URL "%s"', [URL]);
end;

procedure TPJHotLabel.SetCaption(const ACaption : string);
  {"Overridden" write access method for Caption property - checks URL for
  validity if ValidateURL property is true before setting caption. Exception is
  raised (by CheckURL) if URL is checked and found not to be valid}
begin
  if FValidateURL then
    CheckURL(ACaption);
  inherited Caption := ACaption;
end;

function TPJHotLabel.GetCaption : string;
  {"Overridden" read access method for Caption property - reference inherited
  Caption property}
begin
  Result := inherited Caption;
end;

procedure TPJHotLabel.SetValidateURL(const AFlag : Boolean);
  {Write access method for ValidateURL property - ensures that current Caption
  property value is validated if this property is set to true}
begin
  {If we're not changing the property do nothing}
  if AFlag = FValidateURL then Exit;
  {Record new value}
  FValidateURL := AFlag;
  if FValidateURL then
    try
      {We've set property to true so we need to validate the current caption
      as a URL}
      CheckURL(GetCaption);
    except
      on E : EPJURLError do
      begin
        {Caption isn't a valid URL - replace it with default URL and re-raise
        exception so user gets to know about it}
        SetDefaultURL;
        raise;
      end;
    end;
end;

procedure Register;
begin
  RegisterComponents('PJ Stuff', [TPJHotLabel]);
end;

end.
